@@ -2,6 +2,7 @@
 
 package XAO::Web;
 use strict;
+use CGI;
 use XAO::Utils;
 use XAO::Projects;
 use XAO::Objects;
@@ -14,7 +15,7 @@ use XAO::Errors qw(XAO::Web);
 # XAO::Web version number. Hand changed with every release!
 #
 use vars qw($VERSION);
-$VERSION='1.04';
+$VERSION='1.05';
 
 ###############################################################################
 
@@ -99,7 +100,7 @@ Methods of XAO::Web objects include:
 
 ###############################################################################
 
-sub analyze ($@);
+sub analyze ($$;$);
 sub clipboard ($);
 sub config ($);
 sub execute ($%);
@@ -109,10 +110,11 @@ sub sitename ($);
 
 ###############################################################################
 
-=item analize (@)
+=item analyze ($)
 
-Checks how to display the given path. Always returns valid results or
-throws an error if that can't be accomplished.
+Checks how to display the given path (scalar or split up array
+reference). Always returns valid results or throws an error if that
+can't be accomplished.
 
 Returns hash reference:
 
@@ -124,56 +126,88 @@ Returns hash reference:
 
 =cut
  
-sub analyze ($@) {
-    my $self=shift;
-    my $siteconfig=$self->config;
-    my @path=@_;
-    my $path=join('/',@path);
-    my $table=$siteconfig->get('path_mapping_table');
+sub analyze ($$;$) {
+    my ($self,$patharr,$sitename)=@_;
+
+    $patharr=[ split(/\/+/,$patharr) ] unless ref $patharr;
+
+    shift @$patharr while @$patharr && !length($patharr->[0]);
+    unshift(@$patharr,'');
+    my $path=join('/',@$patharr);
 
     ##
     # Looking for the object matching the path.
     #
+    my $siteconfig=$self->config;
+    my $table=$siteconfig->get('path_mapping_table');
     if($table) {
-        for(my $i=@path; $i>=0; $i--) {
-            my $dir=$i ? join('/',@path[0..$i-1]) : '';
-            my $od=$table->{$dir} || $table->{'/'.$dir} || $table->{$dir.'/'} || $table->{'/'.$dir.'/'};
-            next unless $od;
-            my $objname;
-            my %args;
-            if(ref($od)) {
-                $objname=$od->[0];
+        for(my $i=@$patharr; $i>=0; $i--) {
+            my $dir=$i ? join('/',@{$patharr}[0..$i-1]) : '';
+
+            my $od=$table->{$dir} ||
+                   $table->{'/'.$dir} ||
+                   $table->{$dir.'/'} ||
+                   $table->{'/'.$dir.'/'};
+            next unless defined $od;
+
+            ##
+            # If $od is an empty string or an empty array reference --
+            # that means that we need to fall back to default handler
+            # for that path.
+            #
+            # The same happens for 'default' type in a hash reference.
+            #
+            my $rhash;
+            if(ref($od) eq 'HASH') {
+                last if $od->{type} && $od->{type} eq 'default';
+                $rhash=merge_refs($od);
+            }
+            elsif(ref($od) eq 'ARRAY') {
+                last unless @$od;
+                my %args;
                 if(scalar(@{$od})%2 == 1) {
                     %args=@{$od}[1..$#{$od}];
                 }
                 else {
-                    eprint "Odd number of arguments in mapping table, dir=$dir, objname=$objname";
+                    throw XAO::E::Web "analyze - odd number of arguments in the mapping table, dir=$dir, objname=$od->[0]";
                 }
+                $rhash={
+                    type        => 'xaoweb',
+                    objname     => $od->[0],
+                    objargs     => \%args,
+                };
             }
             else {
-                $objname=$od;
+                last unless length($od);
+                $rhash={
+                    type        => 'xaoweb',
+                    objname     => $od,
+                    objargs     => { },
+                };
             }
-            return {
-                objname => $objname,
-                objargs => \%args,
-                path => join('/',@path[$i..$#path]),
-                prefix => $dir,
-                fullpath => $path
-            };
+
+            $rhash->{path}=join('/',@{$patharr}[$i..$#$patharr]);
+            $rhash->{prefix}=$dir;
+            $rhash->{fullpath}=$path;
+
+            return $rhash;
         }
     }
 
     ##
-    # Now looking for exactly matching template. If it matches and
-    # we have some object defined for '/' - then this is our default
-    # object. Otherwise - Page is.
+    # Now looking for exactly matching template and returning Page
+    # object if found.
     #
-    if(XAO::Templates::check(path => $path)) {
+    my $filename=XAO::Templates::filename($path,$sitename);
+    if($filename) {
         return {
-            objname => ($table && $table->{'/'}) ? $table->{'/'} : 'Page',
-            path => $path,
-            fullpath => $path,
-            prefix => join('/',@path[0..($#path-1)])
+            type        => 'xaoweb',
+            objname     => 'Page',
+            objargs     => { },
+            path        => $path,
+            fullpath    => $path,
+            prefix      => join('/',@{$patharr}[0..($#$patharr-1)]),
+            filename    => $filename,
         };
     }
 
@@ -181,10 +215,11 @@ sub analyze ($@) {
     # Nothing was found, returning Default object
     #
     return {
-        objname => ($table && $table->{'/'}) ? $table->{'/'} : 'Default',
-        path => $path,
-        fullpath => $path,
-        prefix => ''
+        type        => 'notfound',
+        objname     => 'Default',
+        path        => $path,
+        fullpath    => $path,
+        prefix      => ''
     };
 }
 
@@ -241,18 +276,29 @@ sub execute ($%) {
     # when page includes something like Redirect object.
     #
     if(defined($header)) {
-        print $header,
-              $pagetext;
+        if(my $r=$args->{apache}) {
+            my $h=$self->config->header_args;
+            while(my ($n,$v)=each %$h) {
+                $r->header_out($n => $v);
+                $r->err_header_out($n => $v);
+            }
+            $r->send_http_header;
+            $r->print($pagetext) unless $r->header_only;
+        }
+        else {
+            print $header,
+                  $pagetext;
+        }
     }
 }
 
 ###############################################################################
 
-=item expand (%) {
+=item expand (%)
 
-Expands given `path' using given `cgi' environment. Returns just the
-text of the page in scalar context and page content plus header content
-in array context.
+Expands given `path' using given `cgi' or 'apache' environment. Returns
+just the text of the page in scalar context and page content plus header
+content in array context.
 
 This is normally used in scripts to execute only a particular template
 and get results of execution.
@@ -269,13 +315,50 @@ Example:
                              MIN_TIME    => time - 86400 * 7,
                          });
 
+See also lower level process() method.
+
 =cut
 
 sub expand ($%) {
     my $self=shift;
     my $args=get_args(\@_);
 
-    my $cgi=$args->{cgi} || throw XAO::E::Web "expand - no 'cgi' given";
+    ##
+    # Processing the page and getting its text
+    #
+    my $pagetext=$self->process($args);
+
+    ##
+    # In scalar context (normal cases) we return only the resulting page
+    # text. In array context (compatibility) we return header as well.
+    #
+    my $siteconfig=$self->config;
+    if(wantarray) {
+        my $header=$siteconfig->header;
+        $siteconfig->cleanup;
+        return ($pagetext,$header);
+    }
+    else {
+        $siteconfig->cleanup;
+        return $pagetext;
+    }
+}
+
+###############################################################################
+
+=item process (%)
+
+Takes the same arguments as the expand() method returning expanded page
+text. Does not clean the site context and should not be called directly
+-- for normal situations either expand() or execute() methods should be
+called.
+
+=cut
+
+sub process ($%) {
+    my $self=shift;
+    my $args=get_args(\@_);
+
     my $siteconfig=$self->config;
     my $sitename=$self->sitename;
 
@@ -298,42 +381,95 @@ sub expand ($%) {
     XAO::PageSupport::reset();
 
     ##
-    # Checking if we have base_url. Guessing it if not.
-    # Ensuring that URL does not end with '/'.
+    # Analyzing the path. We have to do that up here because the object
+    # might specify that we should not touch CGI.
     #
-    if(! $siteconfig->defined("base_url")) {
+    my @path=split(/\//,$path);
+    push(@path,"") unless @path;
+    push(@path,"index.html") if $path =~ /\/$/;
+    my $pd=$args->{pagedesc} || $self->analyze(\@path);
 
-        ##
-        # Base URL should be full path to the start point -
-        # http://host.com in case of rewrite and something like
-        # http://host.com/cgi-bin/xao-apache.pl/sitename in case of
-        # plain CGI usage.
-        #
-        my $url=$cgi->url(-full => 1, -path_info => 0);
-        $url=$1 if $url=~/^(.*)($path)$/;
+    ##
+    # Figuring out current active URL. It might be the same as base_url
+    # and in most cases it is, but it just as well might be different.
+    #
+    # The URL should be full path to the start point -
+    # http://host.com in case of rewrite and something like
+    # http://host.com/cgi-bin/xao-apache.pl/sitename in case of plain
+    # CGI usage.
+    #
+    my $active_url;
+    my $apache=$args->{apache};
+    my $cgi=$args->{cgi};
+    if(!$cgi) {
+        $cgi=$pd->{no_cgi} ? CGI->new('foo=bar') : CGI->new;
+    }
+    if($apache) {
+        $active_url="http://" . $apache->hostname;
+    }
+    else {
+        if(defined($CGI::VERSION) && $CGI::VERSION>=2.80) {
+            $active_url=$cgi->url(-base => 1, -full => 0);
+            my $pinfo=$ENV{PATH_INFO} || '';
+            my $uri=$ENV{REQUEST_URI} || '';
+            $uri=~s/^(.*?)\?.*$/$1/;
+            if($pinfo =~ /^\/\Q$sitename\E(\/.+)?\Q$uri\E/) {
+                # mod_rewrite
+            }
+            elsif($pinfo && $uri =~ /^(.*)\Q$pinfo\E$/) {
+                # cgi
+                $active_url.=$1;
+            }
+            # dprint ">2.8 $active_url";
+        }
+        else {
+            $active_url=$cgi->url(-full => 1, -path_info => 0);
+            $active_url=$1 if $active_url=~/^(.*)(\Q$path\E)$/;
+            # dprint "<2.8 $active_url";
+        }
 
         ##
         # Trying to understand if rewrite module was used or not. If not
         # - adding sitename to the end of guessed URL.
         #
-        if($url =~ /cgi-bin/ || $url =~ /xao-[\w-]+\.pl/) {
-            $url.="/$sitename";
+        if($active_url =~ /cgi-bin/ || $active_url =~ /xao-[\w-]+\.pl/) {
+            $active_url.="/$sitename";
         }
+    }
 
-        ##
-        # Eating extra slashes
-        #
-        chop($url) while $url =~ /\/$/;
-        $url=~s/(?<!:)\/\//\//g;
+    ##
+    # Eating extra slashes
+    #
+    chop($active_url) while $active_url =~ /\/$/;
+    $active_url=~s/(?<!:)\/\//\//g;
 
-        ##
-        # Storing
-        #
-        $siteconfig->put(base_url => $url);
-        $siteconfig->put(base_url_secure => $url);
-        dprint "No base_url defined, sitename=$sitename; assuming base_url=$url";
+    ##
+    # Figuring out secure URL
+    #
+    my $active_url_secure;
+    if($active_url =~ /^http:(\/\/.*)$/) {
+        $active_url_secure='https:' . $1;
+    }
+    elsif($active_url =~ /^https:(\/\/.*)$/) {
+        $active_url_secure=$active_url;
+        $active_url='http:' . $1;
     }
     else {
+        dprint "Wrong active URL ($active_url)";
+        $active_url_secure=$active_url;
+    }
+
+    ##
+    # Storing active URLs
+    #
+    $siteconfig->clipboard->put(active_url => $active_url);
+    $siteconfig->clipboard->put(active_url_secure => $active_url_secure);
+
+    ##
+    # Checking if we have base_url, assuming active_url if not.
+    # Ensuring that URL does not end with '/'.
+    #
+    if($siteconfig->defined("base_url")) {
         my $url=$siteconfig->get('base_url');
         $url=~/^http:/i ||
             throw XAO::E::Web "expand - bad base_url ($url) for sitename=$sitename";
@@ -348,17 +484,25 @@ sub expand ($%) {
         }
         $nu=$url;
         chop($nu) while $nu =~ /\/$/;
-        $siteconfig->put(base_url_secure => $nu) if $nu ne $url;
+        $siteconfig->put(base_url_secure => $nu);
+    }
+    else {
+        $siteconfig->put(base_url => $active_url);
+        $siteconfig->put(base_url_secure => $active_url_secure);
+        dprint "No base_url for sitename '$sitename'; assuming base_url=$active_url, base_url_secure=$active_url_secure";
     }
   
     ##
     # Checking if we're running under mod_perl
     #
-    my $mod_perl=$ENV{MOD_PERL} ? 1 : 0;
+    my $mod_perl=($apache || $ENV{MOD_PERL}) ? 1 : 0;
     $siteconfig->clipboard->put(mod_perl => $mod_perl);
 
     ##
-    # Putting CGI object into site configuration
+    # Putting CGI object into site configuration. The special case is
+    # 'no_cgi' in the path_mapping_table which means that the object is
+    # going to handle CGI arguments itself. It can be useful if it needs
+    # raw query string.
     #
     $siteconfig->embedded('web')->enable_special_access;
     $siteconfig->cgi($cgi);
@@ -368,26 +512,22 @@ sub expand ($%) {
     # Checking for directory index url without trailing slash and
     # redirecting with appended slash if this is the case.
     #
-    my @path=split(/\//,$path);
-    push(@path,"index.html") if $cgi->path_info =~ /\/$/;
     if($path[-1] !~ /\.\w+$/) {
-        my $pd=$self->analyze(@path,'index.html');
+        my $pd=$self->analyze([ @path,'index.html' ]);
+        #use Data::Dumper; dprint "pd=",Dumper($pd);
         if($pd->{objname} ne 'Default') {
-            my $newpath=$siteconfig->get('base_url') . $path . '/';# cgi->url(-full => 1, -path_info => 1)."/";
+            my $newpath=$siteconfig->get('base_url') . $path . '/';
             dprint "Redirecting $path to $newpath";
-            print $cgi->redirect(-url => $newpath),
-                  "Document is really <A HREF=\"$newpath\">here</A>.\n";
-            return;
+            $siteconfig->header_args(
+                -Location   => $newpath,
+                -Status     => 302,
+            );
+            return "Directory index redirection\n";
         }
     }
 
     ##
-    # Checking existence of the page.
-    #
-    my $pd=$self->analyze(@path);
-
-    ##
-    # Separator for error_log :)
+    # Separator for the error_log :)
     #
     my @d=localtime;
     my $date=sprintf("%02u:%02u:%02u %u/%02u/%04u",$d[2],$d[1],$d[0],$d[4]+1,$d[3],$d[5]+1900);
@@ -403,7 +543,17 @@ sub expand ($%) {
     ##
     # We accumulate page content here
     #
-    my $pagetext;
+    my $pagetext='';
+
+    ##
+    # Setting expiration time in the page header to immediate
+    # expiration. If that's not what the page wants -- it can override
+    # these.
+    #
+    $siteconfig->header_args(
+        -expires        => 'now',
+        -cache_control  => 'no-cache',
+    );
 
     ##
     # Do we need to run any objects before executing? A good place to
@@ -411,9 +561,22 @@ sub expand ($%) {
     #
     my $autolist=$siteconfig->get('auto_before');
     if($autolist) {
-        foreach my $objname (keys %{$autolist}) {
-            my $obj=XAO::Objects->new(objname => $objname);
-            $pagetext.=$obj->expand($autolist->{$objname});
+        if(ref($autolist) eq 'ARRAY') {
+            for(my $i=0; $i<@$autolist; $i+=2) {
+                my ($objname,$objargs)=@{$autolist}[$i,$i+1];
+                my $obj=XAO::Objects->new(objname => $objname);
+                $pagetext.=$obj->expand($objargs);
+            }
+        }
+        elsif(ref($autolist) eq 'HASH') {
+            foreach my $objname (keys %{$autolist}) {
+                my $obj=XAO::Objects->new(objname => $objname);
+                $pagetext.=$obj->expand($autolist->{$objname});
+            }
+        }
+        else {
+            throw XAO::E::Web "process - don't know how to handle auto_before ($autolist)," .
+                              " must be a hash or an array reference";
         }
     }
 
@@ -429,14 +592,6 @@ sub expand ($%) {
     };
     $objargs=merge_refs($objargs,$pd->{objargs},$args->{objargs});
 
-    ##
-    # Setting expiration time in the page header to immediate
-    # expiration. If that's not what the page wants -- it can override
-    # these.
-    #
-    $siteconfig->header_args('-expires'         => 'now',
-                             '-Cache-Control'   => 'no-cache');
-
     ##
     # Loading page displaying object and executing it.
     #
@@ -444,16 +599,9 @@ sub expand ($%) {
     $pagetext.=$obj->expand($objargs);
 
     ##
-    # Getting page header
+    # Done!
     #
-    my $header=$siteconfig->header;
-
-    ##
-    # Cleaning up the configuration
-    #
-    $siteconfig->cleanup;
-
-    return wantarray ? ($pagetext,$header) : $pagetext;
+    return $pagetext;
 }
 
 ###############################################################################
@@ -483,10 +631,7 @@ sub new ($%) {
     # Loading or creating site configuration object.
     #
     my $siteconfig=XAO::Projects::get_project($sitename);
-    if($siteconfig) {
-        $siteconfig->cleanup;
-    }
-    else {
+    if(!$siteconfig) {
         ##
         # Creating configuration.
         #
@@ -513,6 +658,12 @@ sub new ($%) {
                                      );
     }
 
+    ##
+    # Cleaning up the configuration. Useful even if it was just created
+    # as it will unlock tables in the database for instance.
+    #
+    $siteconfig->cleanup;
+
     ##
     # If we are given a CGI reference then putting it into the
     # configuration.
