@@ -13,14 +13,24 @@ argument) giving it the following arguments:
 
 =over
 
-=item VERSION
-
-Current XAO::Web package version.
-
 =item COPYRIGHT
 
 Copyright information for XAO::Web.
 
+=item COPYRIGHT.HTML
+
+Copyright information for XAO::Web suitable for HTML, with '&copy;' for
+(C).
+
+=item TITLE
+
+Content of the 'title' argument if there is any or empty string
+otherwise.
+
+=item VERSION
+
+Current XAO::Web package version.
+
 =back
 
 In most cases you would want to extend or override this object or at
@@ -57,16 +67,18 @@ use XAO::Objects;
 use base XAO::Objects->load(objname => 'Web::Page');
 
 use vars qw($VERSION);
-($VERSION)=(q$Id: Footer.pm,v 1.4 2002/03/11 23:45:28 am Exp $ =~ /(\d+\.\d+)/);
+($VERSION)=(q$Id: Footer.pm,v 1.5 2002/12/16 17:30:55 am Exp $ =~ /(\d+\.\d+)/);
 
 sub display ($;%) {
     my $self=shift;
     my $args=get_args(\@_);
 
     my %a=(
-        path => '/bits/page-footer',
-        VERSION => $XAO::Web::VERSION,
-        COPYRIGHT => 'Copyright (C) 2000,2001 XAO, Inc.'
+        path            => '/bits/page-footer',
+        VERSION         => $XAO::Web::VERSION,
+        COPYRIGHT       => 'Copyright (C) 2000-2002 XAO, Inc.',
+        'COPYRIGHT.HTML'=> 'Copyright &copy; 2000-2002 XAO, Inc.',
+        TITLE           => $args->{title} || '',
     );
 
     $self->SUPER::display(merge_refs(\%a,$args));
