@@ -76,7 +76,7 @@ sub display ($;%) {
             %grayed=map { $_ => 1 } values %items;
         }
         else {
-            %grayed=map { $_ => 1 } split(/,/,$args->{grayed});
+            %grayed=map { $_ => 1 } split(/[,;\s]+/,$args->{grayed});
         }
     }
     else {
@@ -88,30 +88,46 @@ sub display ($;%) {
     ##
     # And finally displaying items.
     #
-    my $obj=$self->object;
-    $obj->display(path => "$base/header") if XAO::Templates::check(path => "$base/header");
+    my $page=$self->object;
+    $page->display($args,{
+        path => "$base/header",
+    }) if XAO::Templates::filename("$base/header");
+
     my $first=1;
-    my $sepexists=XAO::Templates::check(path => "$base/separator");
+    my $sepexists=XAO::Templates::filename("$base/separator");
     foreach my $item (sort { ($a =~ /^\d+$/ && $b =~ /^\d+$/)
                                 ? $a <=> $b
                                 : $a cmp $b } keys %items) {
         my $name=$items{$item};
-        $obj->display(path => "$base/separator") if !$first && $sepexists;
+        $page->display(path => "$base/separator") if !$first && $sepexists;
         $first=0;
-        my $path;
+
+        my %params=(
+            NORMAL  => '',
+            ACTIVE  => '',
+            GRAYED  => '',
+        );
+        my $subpath;
         if($grayed{$name}) {
-            $path="grayed";
+            $subpath='grayed';
+            $params{GRAYED}=1;
         }
         elsif(defined($args->{active}) && $name eq $args->{active}) {
-            $path="active";
+            $subpath='active';
+            $params{ACTIVE}=1;
         }
         else {
-            $path="normal";
+            $subpath='normal';
+            $params{NORMAL}=1;
         }
-        $path="$base/item-$name-$path";
-        $obj->display(path => $path);
+        $params{path}="$base/item-$name-$subpath";
+
+        $page->display($args,\%params);
     }
-    $obj->display(path => "$base/footer") if XAO::Templates::check(path => "$base/footer");
+
+    $page->display($args,{
+        path => "$base/footer",
+    }) if XAO::Templates::filename("$base/footer");
 }
 
 ###############################################################################
@@ -124,7 +140,7 @@ Nothing.
 
 =head1 AUTHOR
 
-Copyright (c) 2000-2002 XAO, Inc.
+Copyright (c) 2000-2003 XAO, Inc.
 
 Andrew Maltsev <am@xao.com>.
 
