@@ -19,40 +19,49 @@ XXX - make real documentation!!
 ###############################################################################
 package XAO::DO::Web::Styler;
 use strict;
-use XAO::Utils;
+use XAO::Utils qw(:args fround);
 use XAO::Objects;
 use base XAO::Objects->load(objname => 'Web::Page');
 
+###############################################################################
+
 use vars qw($VERSION);
-($VERSION)=(q$Id: Styler.pm,v 1.4 2002/08/31 00:55:43 am Exp $ =~ /(\d+\.\d+)/);
+($VERSION)=(q$Id: Styler.pm,v 1.9 2003/10/03 04:33:09 am Exp $ =~ /(\d+\.\d+)/);
 
-sub display ($;%)
-{ my $self=shift;
-  my $args=get_args(\@_);
+sub separate_thousands ($);
 
-  ##
-  # Special formatting for special fields.
-  #
-  # number => 1,234,456,789
-  #
-  my $template="<%NUMBER%>" if defined($args->{number});
-  my $number=int($args->{number} || 0);
-  1 while $number=~s/(\d)(\d{3}($|,))/$1,$2/;
+###############################################################################
 
-  ##
-  # dollars => $1'234.78
-  #
-  $template="<%DOLLARS%>" if defined($args->{dollars}) || defined($args->{dollar});
-  my $dollars=sprintf("%.2f",$args->{dollars} || $args->{dollar} || 0);
-  1 while $dollars=~s/(\d)(\d{3}($|,|\.))/$1,$2/;
-  $dollars='$'.$dollars;
+sub display ($;%) {
+    my $self=shift;
+    my $args=get_args(\@_);
 
-  ##
-  # real => 1'234.78
-  #
-  $template='<$REAL$>' if defined($args->{real});
-  my $real=sprintf("%.2f",$args->{real} || 0);
-  1 while $real=~s/(\d)(\d{3}($|,|\.))/$1,$2/;
+    ##
+    # Special formatting for special fields.
+    #
+    # number => 1,234,456,789
+    #
+    my $template="<%NUMBER%>" if defined($args->{number});
+    my $number=int($args->{number} || 0);
+    $number=separate_thousands($number);
+
+    ##
+    # dollars => $1'234.78
+    #
+    $template="<%DOLLARS%>" if defined($args->{dollars}) || defined($args->{dollar});
+    my $dollars=$args->{format}
+                  ? sprintf($args->{format},$args->{dollars} || $args->{dollar} || 0)
+                  : sprintf('%.2f',fround($args->{dollars} || $args->{dollar} || 0,100));
+    $dollars='$' . separate_thousands($dollars);
+
+    ##
+    # real => 1'234.78
+    #
+    $template='<$REAL$>' if defined($args->{real});
+    my $real=$args->{format}
+             ? sprintf($args->{format},$args->{real} || 0)
+             : sprintf("%.2f",$args->{real} || 0);
+    $real=separate_thousands($real);         
 
     ##
     # Percents
@@ -94,6 +103,22 @@ sub display ($;%)
                        , %{$args});
 }
 
+############################## PRIVATE ########################################
+
+sub separate_thousands ($) {
+    my $value=shift;
+
+    return $value unless $value =~ /^(\d+)(\.\d+)?$/;
+
+    my ($i,$f)=($1,$2);
+
+    1 while $i=~s/(\d)(\d{3}($|,))/$1,$2/;
+
+    $i.=$f if defined $f;
+
+    return $i;
+}
+
 ###############################################################################
 1;
 __END__
